# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'runner.ui'
#
# Created by: PyQt5 UI code generator 5.15.4
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(1188, 848)
        self.centralwidget = QtWidgets.QWidget(MainWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.centralwidget)
        self.verticalLayout.setObjectName("verticalLayout")
        self.groupBox_3 = QtWidgets.QGroupBox(self.centralwidget)
        self.groupBox_3.setObjectName("groupBox_3")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.groupBox_3)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.configLayout = QtWidgets.QHBoxLayout()
        self.configLayout.setSizeConstraint(QtWidgets.QLayout.SetDefaultConstraint)
        self.configLayout.setObjectName("configLayout")
        spacerItem = QtWidgets.QSpacerItem(0, 20, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        self.configLayout.addItem(spacerItem)
        self.verticalLayout_3.addLayout(self.configLayout)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.pushButtonClearLog = QtWidgets.QPushButton(self.groupBox_3)
        self.pushButtonClearLog.setObjectName("pushButtonClearLog")
        self.horizontalLayout.addWidget(self.pushButtonClearLog)
        self.pushButtonSaveLog = QtWidgets.QPushButton(self.groupBox_3)
        self.pushButtonSaveLog.setEnabled(True)
        self.pushButtonSaveLog.setObjectName("pushButtonSaveLog")
        self.horizontalLayout.addWidget(self.pushButtonSaveLog)
        self.verticalLayout_3.addLayout(self.horizontalLayout)
        self.line = QtWidgets.QFrame(self.groupBox_3)
        self.line.setFrameShape(QtWidgets.QFrame.HLine)
        self.line.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line.setObjectName("line")
        self.verticalLayout_3.addWidget(self.line)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.textEditAppTrace = QtWidgets.QTextEdit(self.groupBox_3)
        self.textEditAppTrace.setLineWrapMode(QtWidgets.QTextEdit.NoWrap)
        self.textEditAppTrace.setReadOnly(True)
        self.textEditAppTrace.setObjectName("textEditAppTrace")
        self.horizontalLayout_2.addWidget(self.textEditAppTrace)
        self.verticalLayout_2 = QtWidgets.QVBoxLayout()
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.textEditStatusTrace = QtWidgets.QTextEdit(self.groupBox_3)
        self.textEditStatusTrace.setObjectName("textEditStatusTrace")
        self.verticalLayout_2.addWidget(self.textEditStatusTrace)
        spacerItem1 = QtWidgets.QSpacerItem(20, 20, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        self.verticalLayout_2.addItem(spacerItem1)
        self.processListWidget = QtWidgets.QListWidget(self.groupBox_3)
        self.processListWidget.setObjectName("processListWidget")
        self.verticalLayout_2.addWidget(self.processListWidget)
        self.horizontalLayout_2.addLayout(self.verticalLayout_2)
        self.horizontalLayout_2.setStretch(0, 5)
        self.horizontalLayout_2.setStretch(1, 3)
        self.verticalLayout_3.addLayout(self.horizontalLayout_2)
        self.verticalLayout.addWidget(self.groupBox_3)
        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QtWidgets.QMenuBar(MainWindow)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 1188, 23))
        self.menubar.setObjectName("menubar")
        MainWindow.setMenuBar(self.menubar)
        self.statusbar = QtWidgets.QStatusBar(MainWindow)
        self.statusbar.setObjectName("statusbar")
        MainWindow.setStatusBar(self.statusbar)
        self.dockWidget = QtWidgets.QDockWidget(MainWindow)
        self.dockWidget.setFeatures(QtWidgets.QDockWidget.DockWidgetFloatable|QtWidgets.QDockWidget.DockWidgetMovable)
        self.dockWidget.setObjectName("dockWidget")
        self.toolBox = QtWidgets.QWidget()
        self.toolBox.setObjectName("toolBox")
        self.toolBoxLayout = QtWidgets.QVBoxLayout(self.toolBox)
        self.toolBoxLayout.setObjectName("toolBoxLayout")
        self.dockWidget.setWidget(self.toolBox)
        MainWindow.addDockWidget(QtCore.Qt.DockWidgetArea(1), self.dockWidget)

        self.retranslateUi(MainWindow)
        self.pushButtonClearLog.clicked.connect(MainWindow.onClearLog)
        self.pushButtonSaveLog.clicked.connect(MainWindow.onSaveLog)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        MainWindow.setWindowTitle(_translate("MainWindow", "命令启动器"))
        self.groupBox_3.setTitle(_translate("MainWindow", "输出"))
        self.pushButtonClearLog.setText(_translate("MainWindow", "清除log"))
        self.pushButtonSaveLog.setText(_translate("MainWindow", "保存log"))
        self.textEditAppTrace.setPlaceholderText(_translate("MainWindow", "log输出"))
        self.textEditStatusTrace.setPlaceholderText(_translate("MainWindow", "运行指令"))
        self.dockWidget.setWindowTitle(_translate("MainWindow", "工具栏"))
